<?php

namespace Drupal\config_action_report\Controller;

use Drupal\Core\Config\Action\ConfigActionManager;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for Config Action events plugin report.
 */
class ConfigActionReportPluginsController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The Config Action plugin manager.
   */
  protected ConfigActionManager $pluginManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->pluginManager = $container->get('plugin.manager.config_action');
    return $instance;
  }

  /**
   * Displays an Config Action plugin report.
   *
   * @return array
   *   A render array containing an Config Action plugin report.
   */
  public function index(): array {
    // Header.
    $header = [
      'provider' => ['data' => $this->t('Provides'), 'width' => '18%'],
      'label' => ['data' => $this->t('Label'), 'width' => '18%'],
      'plugin_id' => ['data' => $this->t('ID'), 'width' => '18%'],
      'target_entity_type' => ['data' => $this->t('Target entity type'), 'width' => '18%'],
      'entity_types' => ['data' => $this->t('Entity types'), 'width' => '18%'],
      'class' => ['data' => $this->t('Class'), 'width' => '18%'],
    ];

    // Rows.
    $rows = [];
    $definitions = $this->pluginManager->getDefinitions();
    foreach ($definitions as $plugin_id => $definition) {
      // Skip entity methods which are displayed in a dedicated tab.
      if ($definition['class'] === 'Drupal\Core\Config\Action\Plugin\ConfigAction\EntityMethod') {
        continue;
      }

      $row = [];
      $row['provider'] = $definition['provider'];
      $row['label'] = $definition['admin_label'];
      $row['plugin_id'] = [
        'data' => [
          '#markup' => str_replace(':', ':&#x200b;', $plugin_id),
        ],
      ];
      $row['target_entity_type'] = $definition['target_entity_type'] ?? '';
      $row['entity_types'] = ($definition['entity_types'] === ['*'])
        ? $this->t('All')
        : [
            'data' => [
              '#theme' => 'item_list',
              '#items' => $definition['entity_types'],
            ],
          ];
      $row['class'] = [
        'data' => [
          '#markup' => str_replace('\\', '\&#x200b;', $definition['class']),
        ],
      ];

      $rows[] = $row;
    }

    return [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
    ];
  }

}
